@@ -36,7 +36,9 @@ Library
 #include "louis.h"
 #include "transcommon.ci"
 
-static int translateString (void);
+#define MIN(a,b) (((a)<(b))?(a):(b))
+
+static int translateString ();
 static int compbrlStart = 0;
 static int compbrlEnd = 0;
 
@@ -57,9 +59,25 @@ lou_translate (const char *tableList, const widechar
 	       int *inlen, widechar * outbuf, int *outlen,
 	       char *typeform, char *spacing, int *outputPos,
 	       int *inputPos, int *cursorPos, int modex)
+{
+  return trace_translate (tableList, inbufx, inlen, outbuf, outlen,
+			  typeform, spacing, outputPos, inputPos, cursorPos,
+			  NULL, NULL, modex);
+}
+
+int
+trace_translate (const char *tableList, const widechar * inbufx,
+		 int *inlen, widechar * outbuf, int *outlen,
+		 char *typeform, char *spacing, int *outputPos,
+		 int *inputPos, int *cursorPos,
+		 const TranslationTableRule ** rules, int *rulesLen,
+		 int modex)
 {
   int k;
   int goodTrans = 1;
+  if (tableList == NULL || inbufx == NULL || inlen == NULL || outbuf ==
+      NULL || outlen == NULL)
+    return 0;
   if ((modex & otherTrans))
     return other_translate (tableList, inbufx,
 			    inlen, outbuf, outlen,
@@ -124,6 +142,10 @@ lou_translate (const char *tableList, const widechar
     return 0;
   if (!(srcMapping = liblouis_allocMem (alloc_srcMapping, srcmax, destmax)))
     return 0;
+  if (!
+      (prevSrcMapping =
+       liblouis_allocMem (alloc_prevSrcMapping, srcmax, destmax)))
+    return 0;
   for (k = 0; k <= srcmax; k++)
     srcMapping[k] = k;
   srcMapping[srcmax] = srcmax;
@@ -140,15 +162,28 @@ lou_translate (const char *tableList, const widechar
       else
 	memset (destSpacing, '*', destmax);
     }
+  appliedRulesCount = 0;
+  if (rules != NULL && rulesLen != NULL)
+    {
+      appliedRules = rules;
+      maxAppliedRules = *rulesLen;
+    }
+  else
+    {
+      appliedRules = NULL;
+      maxAppliedRules = 0;
+    }
   currentPass = 0;
   if ((mode & pass1Only))
     {
       currentOutput = passbuf1;
+      memcpy (prevSrcMapping, srcMapping, destmax * sizeof (int));
       goodTrans = translateString ();
       currentPass = 5;		/*Certainly > table->numPasses */
     }
   while (currentPass <= table->numPasses && goodTrans)
     {
+      memcpy (prevSrcMapping, srcMapping, destmax * sizeof (int));
       switch (currentPass)
 	{
 	case 0:
@@ -208,8 +243,10 @@ lou_translate (const char *tableList, const widechar
 	  else
 	    outbuf[k] = getCharFromDots (currentOutput[k]);
 	}
-      *inlen = srcMapping[realInlen];
+      *inlen = realInlen;
       *outlen = dest;
+      if (inputPositions != NULL)
+	memcpy (inputPositions, srcMapping, destmax * sizeof (int));
       if (outputPos != NULL)
 	{
 	  int lastpos = 0;
@@ -225,22 +262,79 @@ lou_translate (const char *tableList, const widechar
       memcpy (srcSpacing, destSpacing, srcmax);
       srcSpacing[srcmax] = 0;
     }
-  if (cursorPos != NULL)
-    *cursorPos = cursorPosition;
+  if (cursorPos != NULL && *cursorPos != -1)
+    {
+      if (outputPos != NULL)
+	*cursorPos = outputPos[*cursorPos];
+      else
+	*cursorPos = cursorPosition;
+    }
+  if (rulesLen != NULL)
+    *rulesLen = appliedRulesCount;
   return goodTrans;
 }
 
+int EXPORT_CALL
+lou_translatePrehyphenated (const char *tableList,
+			    const widechar * inbufx, int *inlen,
+			    widechar * outbuf, int *outlen,
+			    char *typeform, char *spacing,
+			    int *outputPos, int *inputPos, int *cursorPos,
+			    char *inputHyphens, char *outputHyphens,
+			    int modex)
+{
+  int rv = 1;
+  int *alloc_inputPos = NULL;
+  if (inputHyphens != NULL)
+    {
+      if (outputHyphens == NULL)
+	return 0;
+      if (inputPos == NULL)
+	{
+	  if ((alloc_inputPos = malloc (*outlen * sizeof (int))) == NULL)
+	    outOfMemory ();
+	  inputPos = alloc_inputPos;
+	}
+    }
+  if (lou_translate (tableList, inbufx, inlen, outbuf, outlen, typeform,
+		     spacing, outputPos, inputPos, cursorPos, modex))
+    {
+      if (inputHyphens != NULL)
+	{
+	  int inpos = 0;
+	  int outpos;
+	  for (outpos = 0; outpos < *outlen; outpos++)
+	    {
+	      int new_inpos = inputPos[outpos];
+	      if (new_inpos < inpos)
+		{
+		  rv = 0;
+		  break;
+		}
+	      if (new_inpos > inpos)
+		outputHyphens[outpos] = inputHyphens[new_inpos];
+	      else
+		outputHyphens[outpos] = '0';
+	      inpos = new_inpos;
+	    }
+	}
+    }
+  if (alloc_inputPos != NULL)
+    free (alloc_inputPos);
+  return rv;
+}
+
 static TranslationTableOpcode indicOpcode;
 static const TranslationTableRule *indicRule;
 static int dontContract = 0;
 
-static int doCompbrl (void);
+static int doCompbrl ();
 
 static int
 hyphenate (const widechar * word, int wordSize, char *hyphens)
 {
-  widechar prepWord[MAXSTRING];
-  int i, j, k;
+  widechar *prepWord;
+  int i, k, limit;
   int stateNum;
   widechar ch;
   HyphenationState *statesArray = (HyphenationState *)
@@ -251,54 +345,68 @@ hyphenate (const widechar * word, int wordSize, char *hyphens)
   int patternOffset;
   if (!table->hyphenStatesArray || (wordSize + 3) > MAXSTRING)
     return 0;
-  j = 0;
-  prepWord[j++] = '.';
+  prepWord = (widechar *) calloc(wordSize+3, sizeof(widechar));
+  /* prepWord is of the format ".hello."
+  * hyphens is the length of the word "hello" "00000" */
+  prepWord[0] = '.';
   for (i = 0; i < wordSize; i++)
-    prepWord[j++] = (findCharOrDots (word[i], 0))->lowercase;
-  prepWord[j++] = '.';
-  prepWord[j] = 0;
-  for (i = 0; i < j; i++)
-    hyphens[i] = '0';
+    {
+      prepWord[i+1] = (findCharOrDots (word[i], 0))->lowercase;
+      hyphens[i] = '0';
+    }
+  prepWord[wordSize+1] = '.';
+
   /* now, run the finite state machine */
   stateNum = 0;
-  for (i = 0; i < j; i++)
+
+  // we need to walk all of ".hello."
+  for (i = 0; i < wordSize+2; i++)
     {
       ch = prepWord[i];
       while (1)
-	{
-	  if (stateNum == 0xffff)
-	    {
-	      stateNum = 0;
-	      goto nextLetter;
-	    }
-	  currentState = &statesArray[stateNum];
-	  if (currentState->trans.offset)
-	    {
-	      transitionsArray = (HyphenationTrans *) &
-		table->ruleArea[currentState->trans.offset];
-	      for (k = 0; k < currentState->numTrans; k++)
-		if (transitionsArray[k].ch == ch)
-		  {
-		    stateNum = transitionsArray[k].newState;
-		    goto stateFound;
-		  }
-	    }
-	  stateNum = currentState->fallbackState;
-	}
+        {
+          if (stateNum == 0xffff)
+            {
+              stateNum = 0;
+              goto nextLetter;
+            }
+          currentState = &statesArray[stateNum];
+          if (currentState->trans.offset)
+            {
+              transitionsArray = (HyphenationTrans *) &
+                table->ruleArea[currentState->trans.offset];
+              for (k = 0; k < currentState->numTrans; k++)
+                {
+                  if (transitionsArray[k].ch == ch)
+                    {
+                      stateNum = transitionsArray[k].newState;
+                      goto stateFound;
+                    }
+                }
+            }
+            stateNum = currentState->fallbackState;
+        }
     stateFound:
       currentState = &statesArray[stateNum];
       if (currentState->hyphenPattern)
-	{
-	  hyphenPattern =
-	    (char *) &table->ruleArea[currentState->hyphenPattern];
-	  patternOffset = i + 1 - strlen (hyphenPattern);
-	  for (k = 0; hyphenPattern[k]; k++)
-	    if (hyphens[patternOffset + k] < hyphenPattern[k])
-	      hyphens[patternOffset + k] = hyphenPattern[k];
-	}
+        {
+          hyphenPattern = (char *) &table->ruleArea[currentState->hyphenPattern];
+          patternOffset = i+1 - strlen(hyphenPattern);
+
+          /* Need to ensure that we don't overrun hyphens,
+          * in some cases hyphenPattern is longer than the remaining letters,
+          * and if we write out all of it we would have overshot our buffer. */
+          limit = MIN(strlen(hyphenPattern), wordSize-patternOffset);
+          for (k = 0; k<limit; k++)
+            {
+              if (hyphens[patternOffset + k] < hyphenPattern[k])
+                hyphens[patternOffset + k] = hyphenPattern[k];
+            }
+        }
     nextLetter:;
     }
   hyphens[wordSize] = 0;
+  free(prepWord);
   return 1;
 }
 
@@ -344,26 +452,26 @@ for_updatePositions (const widechar * outChars, int inLength, int outLength)
 	  for (k = 0; k < outLength; k++)
 	    {
 	      if (inputPositions != NULL)
-		inputPositions[dest + k] = srcMapping[src];
+		srcMapping[dest + k] = prevSrcMapping[src];
 	      if (outputPositions != NULL)
-		outputPositions[srcMapping[src + k]] = dest;
+		outputPositions[prevSrcMapping[src + k]] = dest;
 	    }
 	  for (k = outLength; k < inLength; k++)
 	    if (outputPositions != NULL)
-	      outputPositions[srcMapping[src + k]] = dest;
+	      outputPositions[prevSrcMapping[src + k]] = dest;
 	}
       else
 	{
 	  for (k = 0; k < inLength; k++)
 	    {
 	      if (inputPositions != NULL)
-		inputPositions[dest + k] = srcMapping[src];
+		srcMapping[dest + k] = prevSrcMapping[src];
 	      if (outputPositions != NULL)
-		outputPositions[srcMapping[src + k]] = dest;
+		outputPositions[prevSrcMapping[src + k]] = dest;
 	    }
 	  for (k = inLength; k < outLength; k++)
 	    if (inputPositions != NULL)
-	      inputPositions[dest + k] = srcMapping[src];
+	      srcMapping[dest + k] = prevSrcMapping[src];
 	}
     }
   dest += outLength;
@@ -371,39 +479,47 @@ for_updatePositions (const widechar * outChars, int inLength, int outLength)
 }
 
 static int
-syllableBreak (void)
+syllableBreak ()
 {
-  int wordStart;
-  int wordEnd;
-  int k;
-  char hyphens[MAXSTRING];
+  int wordStart=0;
+  int wordEnd=0;
+  int wordSize=0;
+  int k=0;
+  char *hyphens = NULL;
   for (wordStart = src; wordStart >= 0; wordStart--)
     if (!((findCharOrDots (currentInput[wordStart], 0))->attributes &
-	  CTC_Letter))
+          CTC_Letter))
       {
-	wordStart++;
-	break;
+        wordStart++;
+        break;
       }
   if (wordStart < 0)
     wordStart = 0;
   for (wordEnd = src; wordEnd < srcmax; wordEnd++)
     if (!((findCharOrDots (currentInput[wordEnd], 0))->attributes &
-	  CTC_Letter))
+          CTC_Letter))
       {
-	wordEnd--;
-	break;
+        wordEnd--;
+        break;
       }
-  if (!hyphenate (&currentInput[wordStart], wordEnd - wordStart, hyphens))
-    return 0;
-/* If the number at the beginning of the syllable is odd or all 
-* numbers are even there is no syllable break. Otherwise there is.*/
-  k = src - wordStart;
-  if (hyphens[k] & 1)
-    return 0;
-  k++;
-  for (; k < (src - wordStart + transCharslen); k++)
+  if (wordEnd == srcmax) wordEnd--;
+  /* At this stage wordStart is the 0 based index of the first letter in the word,
+  * wordEnd is the 0 based index of the last letter in the word.
+  * example: "hello" wordstart=0, wordEnd=4. */
+  wordSize=wordEnd-wordStart+1;
+  hyphens = (char *) calloc(wordSize+1, sizeof(char));
+  if (!hyphenate (&currentInput[wordStart], wordSize, hyphens))
+    {
+      free(hyphens);
+      return 0;
+    }
+  for (k = src - wordStart+1; k < (src - wordStart + transCharslen); k++) 
     if (hyphens[k] & 1)
-      return 1;
+      {
+        free(hyphens);
+        return 1;
+    }
+  free(hyphens);
   return 0;
 }
 
@@ -412,7 +528,7 @@ static widechar before, after;
 static TranslationTableCharacterAttributes beforeAttributes;
 static TranslationTableCharacterAttributes afterAttributes;
 static void
-setBefore (void)
+setBefore ()
 {
   if (src >= 2 && currentInput[src - 1] == ENDSEGMENT)
     before = currentInput[src - 2];
@@ -509,7 +625,7 @@ markWords (const TranslationTableOffset * offset)
 }
 
 static int
-insertIndicators (void)
+insertIndicators ()
 {
 /*Insert italic, bold, etc. indicators before words*/
   int typeMark;
@@ -561,7 +677,7 @@ validMatch ()
   TranslationTableCharacterAttributes prevAttr = 0;
   int k;
   int kk = 0;
-  unsigned short mask;
+  unsigned short mask = 0;
   if (!transCharslen)
     return 0;
   switch (transOpcode)
@@ -590,28 +706,35 @@ validMatch ()
       if (k == src)
 	prevAttr = currentInputChar->attributes;
       ruleChar = findCharOrDots (transRule->charsdots[kk++], 0);
-      if ((currentInputChar->lowercase != ruleChar->lowercase)
-	  || (typebuf != NULL && (typebuf[src] & capsemph) == 0 &&
-	      (typebuf[k] & mask) != (typebuf[src] & mask))
-	  || (k != (src + 1) && (prevAttr &
+      if ((currentInputChar->lowercase != ruleChar->lowercase))
+	return 0;
+      if (typebuf != NULL && (typebuf[src] & capsemph) == 0 &&
+	  (typebuf[k] & mask) != (typebuf[src] & mask))
+	return 0;
+      if (currentInputChar->attributes != CTC_Letter)
+	{
+	  if (k != (src + 1) && (prevAttr &
 				 CTC_Letter)
 	      && (currentInputChar->attributes & CTC_Letter)
 	      &&
 	      ((currentInputChar->
-		attributes & (CTC_LowerCase | CTC_UpperCase)) !=
-	       (prevAttr & (CTC_LowerCase | CTC_UpperCase)))))
-	return 0;
+		attributes & (CTC_LowerCase | CTC_UpperCase |
+			      CTC_Letter)) !=
+	       (prevAttr & (CTC_LowerCase | CTC_UpperCase | CTC_Letter))))
+	    return 0;
+	}
       prevAttr = currentInputChar->attributes;
     }
   return 1;
 }
 
 static int
-checkMultCaps (void)
+checkMultCaps ()
 {
   int k;
   for (k = 0; k < table->lenBeginCaps; k++)
-    if (!checkAttr (currentInput[src + k], CTC_UpperCase, 0))
+    if (k >= srcmax - src ||
+	!checkAttr (currentInput[src + k], CTC_UpperCase, 0))
       return 0;
   return 1;
 }
@@ -677,7 +800,7 @@ endEmphasis (const TranslationTableOffset * offset)
 }
 
 static int
-doCompEmph (void)
+doCompEmph ()
 {
   int endEmph;
   for (endEmph = src; (typebuf[endEmph] & computer_braille) && endEmph
@@ -994,7 +1117,7 @@ insertBrailleIndicators (int finish)
 }
 
 static int
-onlyLettersBehind (void)
+onlyLettersBehind ()
 {
   /* Actually, spaces, then letters */
   int k;
@@ -1016,7 +1139,7 @@ onlyLettersBehind (void)
 }
 
 static int
-onlyLettersAhead (void)
+onlyLettersAhead ()
 {
   /* Actullly, spaces, then letters */
   int k;
@@ -1038,14 +1161,14 @@ onlyLettersAhead (void)
 }
 
 static int
-noCompbrlAhead (void)
+noCompbrlAhead ()
 {
   int start = src + transCharslen;
   int end;
   int curSrc;
   if (start >= srcmax)
     return 1;
-  while (checkAttr (currentInput[start], CTC_Space, 0) && start < srcmax)
+  while (start < srcmax && checkAttr (currentInput[start], CTC_Space, 0))
     start++;
   if (start == srcmax || (transOpcode == CTO_JoinableWord && (!checkAttr
 							      (currentInput
@@ -1060,7 +1183,7 @@ noCompbrlAhead (void)
 							       0))))
     return 1;
   end = start;
-  while (!checkAttr (currentInput[end], CTC_Space, 0) && end < srcmax)
+  while (end < srcmax && !checkAttr (currentInput[end], CTC_Space, 0))
     end++;
   if ((mode & (compbrlAtCursor | compbrlLeftCursor)) && cursorPosition
       >= start && cursorPosition < end)
@@ -1125,7 +1248,7 @@ noCompbrlAhead (void)
 static widechar const *repwordStart;
 static int repwordLength;
 static int
-isRepeatedWord (void)
+isRepeatedWord ()
 {
   int start;
   if (src == 0 || !checkAttr (currentInput[src - 1], CTC_Letter, 0))
@@ -1147,7 +1270,7 @@ isRepeatedWord (void)
 }
 
 static void
-for_selectRule (void)
+for_selectRule ()
 {
 /*check for valid Translations. Return value is in transRule. */
   int length = srcmax - src;
@@ -1446,11 +1569,11 @@ undefinedCharacter (widechar c)
   if ((dest + strlen (display)) > destmax)
     return 0;
   if (outputPositions != NULL)
-    outputPositions[srcMapping[src]] = dest;
+    outputPositions[prevSrcMapping[src]] = dest;
   for (k = 0; k < strlen (display); k++)
     {
       if (inputPositions != NULL)
-	inputPositions[dest] = srcMapping[src];
+	srcMapping[dest] = prevSrcMapping[src];
       currentOutput[dest++] = getDotsForChar (display[k]);
     }
   return 1;
@@ -1496,7 +1619,7 @@ putCharacters (const widechar * characters, int count)
 }
 
 static int
-doCompbrl (void)
+doCompbrl ()
 {
 /*Handle strings containing substrings defined by the compbrl opcode*/
   int stringEnd;
@@ -1572,7 +1695,7 @@ doCompTrans (int start, int end)
 }
 
 static int
-doNocont (void)
+doNocont ()
 {
 /*Handle strings containing substrings defined by the nocont opcode*/
   if (checkAttr (currentInput[src], CTC_Space, 0) || dontContract
@@ -1593,7 +1716,7 @@ doNocont (void)
 }
 
 static int
-markSyllables (void)
+markSyllables ()
 {
   int k;
   int syllableMarker = 0;
@@ -1683,7 +1806,7 @@ markSyllables (void)
 }
 
 static int
-translateString (void)
+translateString ()
 {
 /*Main translation routine */
   int k;
@@ -1713,6 +1836,8 @@ translateString (void)
       if (!insertIndicators ())
 	goto failure;
       for_selectRule ();
+      if (appliedRules != NULL && appliedRulesCount < maxAppliedRules)
+	appliedRules[appliedRulesCount++] = transRule;
       srcIncremented = 1;
       prevSrc = src;
       switch (transOpcode)	/*Rules that pre-empt context and swap */
@@ -1731,6 +1856,8 @@ translateString (void)
 	switch (transOpcode)
 	  {
 	  case CTO_Context:
+	    if (appliedRules != NULL && appliedRulesCount < maxAppliedRules)
+	      appliedRules[appliedRulesCount++] = transRule;
 	    if (!passDoAction ())
 	      goto failure;
 	    if (endReplace == src)
@@ -1819,15 +1946,16 @@ translateString (void)
 		}
 	      else
 		{
-		  for (k = src; k < (src + transCharslen); k++)
+		  for (k = 0; k < transCharslen; k++)
 		    {
-		      if (!putCharacter (currentInput[k]))
+		      if (!putCharacter (currentInput[src]))
 			goto failure;
+		      src++;
 		    }
 		}
 	      if (cursorStatus == 2)
 		cursorStatus = 1;
-	      else
+	      else if (transRule->dotslen)
 		src += transCharslen;
 	    }
 	  break;
@@ -1853,7 +1981,7 @@ translateString (void)
 		  {
 		    int tcc;
 		    for (tcc = 0; tcc < transCharslen; tcc++)
-		      outputPositions[srcMapping[src + tcc]] = dest - 1;
+		      outputPositions[prevSrcMapping[src + tcc]] = dest - 1;
 		  }
 		if (!cursorStatus && src <= cursorPosition
 		    && cursorPosition < src + transCharslen)
@@ -1882,7 +2010,7 @@ translateString (void)
 		  {
 		    int tcc;
 		    for (tcc = 0; tcc < transCharslen; tcc++)
-		      outputPositions[srcMapping[src + tcc]] = dest - 1;
+		      outputPositions[prevSrcMapping[src + tcc]] = dest - 1;
 		  }
 		if (!cursorStatus && src <= cursorPosition
 		    && cursorPosition < src + transCharslen)
@@ -1946,7 +2074,8 @@ lou_hyphenate (const char *tableList, const widechar
   int wordStart;
   int wordEnd;
   table = lou_getTable (tableList);
-  if (table == NULL || table->hyphenStatesArray == 0 || inlen >= HYPHSTRING)
+  if (table == NULL || inbuf == NULL || hyphens
+      == NULL || table->hyphenStatesArray == 0 || inlen >= HYPHSTRING)
     return 0;
   if (mode != 0)
     {
@@ -2044,6 +2173,8 @@ lou_dotsToChar (const char *tableList, widechar * inbuf, widechar * outbuf,
 {
   int k;
   widechar dots;
+  if (tableList == NULL || inbuf == NULL || outbuf == NULL)
+    return 0;
   if ((mode & otherTrans))
     return other_dotsToChar (tableList, inbuf, outbuf, length, mode);
   table = lou_getTable (tableList);
@@ -2064,6 +2195,8 @@ lou_charToDots (const char *tableList, const widechar * inbuf, widechar *
 		outbuf, int length, int mode)
 {
   int k;
+  if (tableList == NULL || inbuf == NULL || outbuf == NULL)
+    return 0;
   if ((mode & otherTrans))
     return other_charToDots (tableList, inbuf, outbuf, length, mode);
 
