@@ -6,6 +6,8 @@ $|++;
 # Test all tables with lou_checktable.
 #
 # Copyright (C) 2010 by Swiss Library for the Blind, Visually Impaired and Print Disabled
+# Copyright (C) 2012-2013 Mesar Hameed <mhameed @ src.gnome.org>
+# 
 #
 # Copying and distribution of this file, with or without modification,
 # are permitted in any medium without royalty provided the copyright
@@ -21,9 +23,8 @@ my $timeout = 120; # seconds
 my $tablesdir = (split(',', $ENV{LOUIS_TABLEPATH}))[0];
 
 # get all the tables from the tables directory
-my @tables = glob("$tablesdir/*.[cu]tb $tablesdir/*.cti $tablesdir/*.dis");
-# filter tables that only work when included inside others
-@tables = grep(!/countries.cti|compress.ctb|corrections.ctb|core.[cu]tb|-translation.ctb/, @tables);
+my @tables = glob("$tablesdir/*.{utb,ctb}");
+
 
 foreach my $table (@tables) {
     if (my $pid = fork) {
@@ -35,7 +36,7 @@ foreach my $table (@tables) {
     } else {
 	die "cannot fork: $!" unless defined($pid);
 	alarm $timeout;
-	exec ("lou_checktable $table --quiet");
+	exec ("../tools/lou_checktable $table --quiet");
 	die "Exec of lou_checktable failed: $!";
     }
 }
