@@ -25,7 +25,7 @@ Library
    */
 
 #ifdef HAVE_CONFIG_H
-# include "config.h"
+#include "config.h"
 #endif
 
 #include <stdio.h>
@@ -53,7 +53,7 @@ static void
 print_help (void)
 {
   printf ("\
-Usage: %s [OPTION]\n", program_name);
+Usage: %s [OPTIONS]\n", program_name);
   
   fputs ("\
 This program tests every capability of the liblouis library. It is\n\
@@ -64,8 +64,14 @@ completely interactive. \n\n", stdout);
   -v, --version       display version information and exit\n", stdout);
 
   printf ("\n");
-  printf ("\
-Report bugs to <%s>.\n", PACKAGE_BUGREPORT);
+  printf ("Report bugs to %s.\n", PACKAGE_BUGREPORT);
+
+#ifdef PACKAGE_PACKAGER_BUG_REPORTS
+  printf ("Report %s bugs to: %s\n", PACKAGE_PACKAGER, PACKAGE_PACKAGER_BUG_REPORTS);
+#endif
+#ifdef PACKAGE_URL
+  printf ("%s home page: <%s>\n", PACKAGE_NAME, PACKAGE_URL);
+#endif
 }
 
 #define BUFSIZE 256
@@ -78,7 +84,7 @@ static int showPositions = 0;
 static int minimalist = 0;
 static int outputSize = BUFSIZE;
 static int showSizes = 0;
-static int enteredCursorPos;
+static int enteredCursorPos = -1;
 static unsigned int mode;
 static char table[BUFSIZE];
 static char emphasis[BUFSIZE];
@@ -94,7 +100,10 @@ getInput (void)
   fgets (inputBuffer, sizeof (inputBuffer), stdin);
   inputLength = strlen (inputBuffer) - 1;
   if (inputLength < 0)		/*EOF on script */
-    exit (0);
+    {
+      lou_free();
+      exit (EXIT_SUCCESS);
+    }
   inputBuffer[inputLength] = 0;
   return inputLength;
 }
@@ -195,13 +204,13 @@ getCommands (void)
 	  printf ("(Enter an x to cancel emphasis.)\n");
 	  printf ("Enter an emphasis string: ");
 	  getInput ();
-	  strcpy (emphasis, inputBuffer);
+	  strcpy (enteredEmphasis, inputBuffer);
 	  break;
 	case 's':
 	  printf ("(Enter an x to cancel spacing.)\n");
 	  printf ("Enter a spacing string: ");
 	  getInput ();
-	  strcpy (spacing, inputBuffer);
+	  strcpy (enteredSpacing, inputBuffer);
 	  break;
 	case 'h':
 	  printf ("Commands: action\n");
@@ -223,7 +232,8 @@ getCommands (void)
 	  paramLetters ();
 	  break;
 	case 'q':
-	  exit (0);
+	  lou_free();
+	  exit (EXIT_SUCCESS);
 	case 'f':
 	  printf ("Do only forward translation");
 	  forwardOnly = getYN ();
@@ -256,6 +266,7 @@ getCommands (void)
 int
 main (int argc, char **argv)
 {
+  char *charbuf;
   widechar inbuf[BUFSIZE];
   widechar transbuf[BUFSIZE];
   widechar outbuf[BUFSIZE];
@@ -264,7 +275,7 @@ main (int argc, char **argv)
   int inlen;
   int translen;
   int outlen;
-  int cursorPos;
+  int cursorPos = -1;
   int realInlen = 0;
   int k;
   int optc;
@@ -316,25 +327,28 @@ main (int argc, char **argv)
 	    inlen = getInput ();
 	    if (inlen == 0)
 	      break;
-	    for (realInlen = 0; realInlen < inlen; realInlen++)
-	      inbuf[realInlen] = inputBuffer[realInlen];
+	    if (!(realInlen = extParseChars (inputBuffer, inbuf)))
+	      break;
+	    inlen = realInlen;
 	    if (!lou_translateString (table, inbuf, &inlen, transbuf,
 				      &translen, NULL, NULL, 0))
 	      break;
 	    transbuf[translen] = 0;
 	    printf ("Translation:\n");
-	    for (k = 0; k < translen; k++)
-	      printf ("%c", transbuf[k]);
-	    printf ("\n");
+	    charbuf = showString (transbuf, translen);
+	    k = strlen (charbuf) - 1;
+	    charbuf[k] = 0;
+	    printf ("%s\n", &charbuf[1]);
 	    if (showSizes)
 	      printf ("input length = %d; output length = %d\n", inlen,
 		      translen);
 	    lou_backTranslateString (table, transbuf, &translen, outbuf,
 				     &outlen, NULL, NULL, 0);
 	    printf ("Back-translation:\n");
-	    for (k = 0; k < outlen; k++)
-	      printf ("%c", outbuf[k]);
-	    printf ("\n");
+	    charbuf = showString (outbuf, outlen);
+	    k = strlen (charbuf) - 1;
+	    charbuf[k] = 0;
+	    printf ("%s\n", &charbuf[1]);
 	    if (showSizes)
 	      printf ("input length = %d; output length = %d.\n", translen,
 		      outlen);
@@ -359,14 +373,16 @@ main (int argc, char **argv)
 	    outlen = outputSize;
 	    if (backOnly)
 	      {
-		for (translen = 0; translen < inlen; translen++)
-		  transbuf[translen] = inputBuffer[translen];
+	    if (!(translen = extParseChars (inputBuffer, transbuf)))
+	      break;
+	    inlen = realInlen;
 	      }
 	    else
 	      {
 		translen = outputSize;
-		for (realInlen = 0; realInlen < inlen; realInlen++)
-		  inbuf[realInlen] = inputBuffer[realInlen];
+	    if (!(realInlen = extParseChars (inputBuffer, inbuf)))
+	      break;
+	    inlen = realInlen;
 		if (!lou_translate (table, inbuf, &inlen, transbuf,
 				    &translen, emphasis, spacing,
 				    &outputPos[0], &inputPos[0], &cursorPos,
@@ -381,9 +397,10 @@ main (int argc, char **argv)
 		else
 		  {
 		    printf ("Translation:\n");
-		    for (k = 0; k < translen; k++)
-		      printf ("%c", transbuf[k]);
-		    printf ("\n");
+		    charbuf = showString (transbuf, translen);
+		    k = strlen (charbuf) - 1;
+		    charbuf[k] = 0;
+		    printf ("%s\n", &charbuf[1]);
 		    if (showSizes)
 		      printf ("input length = %d; output length = %d\n",
 			      inlen, translen);
@@ -391,8 +408,6 @@ main (int argc, char **argv)
 	      }
 	    if (cursorPos != -1)
 	      printf ("Cursor position: %d\n", cursorPos);
-	    if (enteredEmphasis[0])
-	      printf ("Returned emphasis: %s\n", emphasis);
 	    if (enteredSpacing[0])
 	      printf ("Returned spacing: %s\n", spacing);
 	    if (showPositions)
@@ -414,16 +429,15 @@ main (int argc, char **argv)
 					&inputPos[0], &cursorPos, mode))
 		  break;
 		printf ("Back-translation:\n");
-		for (k = 0; k < outlen; k++)
-		  printf ("%c", outbuf[k]);
-		printf ("\n");
+		charbuf = showString (outbuf, outlen);		k = 
+		strlen (charbuf) - 1;
+		charbuf[k] = 0;
+		printf ("%s\n", &charbuf[1]);
 		if (showSizes)
 		  printf ("input length = %d; output length = %d\n",
 			  translen, outlen);
 		if (cursorPos != -1)
 		  printf ("Cursor position: %d\n", cursorPos);
-		if (enteredEmphasis[0])
-		  printf ("Returned emphasis: %s\n", emphasis);
 		if (enteredSpacing[0])
 		  printf ("Returned spacing: %s\n", spacing);
 		if (showPositions)
@@ -452,5 +466,5 @@ main (int argc, char **argv)
 	  }
     }
   lou_free ();
-  return 0;
+  exit (EXIT_SUCCESS);
 }
