@@ -54,7 +54,7 @@ static void
 print_help (void)
 {
   printf ("\
-Usage: %s [OPTION] TABLE\n", program_name);
+Usage: %s [OPTIONS] TABLE[,TABLE,...]\n", program_name);
   
   fputs ("\
 Test a Braille translation table. If the table contains errors,\n\
@@ -65,11 +65,17 @@ option.\n", stdout);
   fputs ("\
   -h, --help          display this help and exit\n\
   -v, --version       display version information and exit\n\
-  -q, --quiet         do not write to standard error if there are no errors.", stdout);
+  -q, --quiet         do not write to standard error if there are no errors.\n", stdout);
 
   printf ("\n");
-  printf ("\
-Report bugs to <%s>.\n", PACKAGE_BUGREPORT);
+  printf ("Report bugs to %s.\n", PACKAGE_BUGREPORT);
+
+#ifdef PACKAGE_PACKAGER_BUG_REPORTS
+  printf ("Report %s bugs to: %s\n", PACKAGE_PACKAGER, PACKAGE_PACKAGER_BUG_REPORTS);
+#endif
+#ifdef PACKAGE_URL
+  printf ("%s home page: <%s>\n", PACKAGE_NAME, PACKAGE_URL);
+#endif
 }
 
 int
