@@ -32,10 +32,11 @@
 #include <stdlib.h>
 #include <getopt.h>
 #include "liblouis.h"
+#include "louis.h"
 #include "progname.h"
 #include "version-etc.h"
 
-#define BUFSIZE 2048
+#define BUFSIZE MAXSTRING - 4
 
 static int forward_flag = 0;
 static int backward_flag = 0;
@@ -57,37 +58,37 @@ const char version_etc_copyright[] =
 static void 
 translate_input (int forward_translation, char *table_name)
 {
+  char charbuf[BUFSIZE];
+  char *outputbuf;
   widechar inbuf[BUFSIZE];
   widechar transbuf[BUFSIZE];
   int inlen;
   int translen;
-  int ch = 0;
   int k;
+  int ch = 0;
   int result;
-
   while (1)
     {
       translen = BUFSIZE;
-      inlen = 0;
-      while ((ch = getchar ()) != '\n' && inlen < BUFSIZE)
-	inbuf[inlen++] = ch;
-      if (ch == EOF)
+      k = 0;
+      while ((ch = getchar ()) != '\n' && ch != EOF && k < BUFSIZE)
+	charbuf[k++] = ch;
+      if (ch == EOF && k == 0)
 	break;
-      inbuf[inlen] = 0;
+      charbuf[k] = 0;
+      inlen = extParseChars (charbuf, inbuf);
       if (forward_translation) 
-	{
 	  result = lou_translateString (table_name, inbuf, &inlen,
 				      transbuf, &translen, NULL, NULL, 0);
-	}
       else 
 	result = lou_backTranslateString (table_name, inbuf, &inlen,
 					  transbuf, &translen, NULL, NULL, 0);
       if (!result)
 	break;
-      transbuf[translen] = 0;
-      for (k = 0; k < translen; k++)
-	printf ("%c", transbuf[k]);
-      printf ("\n");
+      outputbuf = showString (transbuf, translen);
+      k = strlen (outputbuf) - 1;
+      outputbuf[k] = 0;
+      printf ("%s\n", &outputbuf[1]);
     }
   lou_free ();
 }
@@ -96,7 +97,7 @@ static void
 print_help (void)
 {
   printf ("\
-Usage: %s [OPTION] TABLE\n", program_name);
+Usage: %s [OPTIONS] TABLE[,TABLE,...]\n", program_name);
   
   fputs ("\
 Translate whatever is on standard input and print it on standard\n\
@@ -111,8 +112,14 @@ Braille translation and back-translation.\n\n", stdout);
                       If neither -f nor -b are specified forward translation\n\
                       is assumed\n", stdout);
   printf ("\n");
-  printf ("\
-Report bugs to <%s>.\n", PACKAGE_BUGREPORT);
+  printf ("Report bugs to %s.\n", PACKAGE_BUGREPORT);
+
+#ifdef PACKAGE_PACKAGER_BUG_REPORTS
+  printf ("Report %s bugs to: %s\n", PACKAGE_PACKAGER, PACKAGE_PACKAGER_BUG_REPORTS);
+#endif
+#ifdef PACKAGE_URL
+  printf ("%s home page: <%s>\n", PACKAGE_NAME, PACKAGE_URL);
+#endif
 }
 
 int
